@@ -35,6 +35,7 @@
 #include "conf.h"
 #include "dnswire.h"
 #include "dnspacket.h"
+#include "dnsio.h"
 #include "gdnsd/log.h"
 #include "gdnsd/prcu-priv.h"
 
@@ -136,12 +137,16 @@ static void udp_sock_opts_v6(const int sock) {
         log_fatal("Failed to set IPV6_RECVPKTINFO on UDP socket: %s", logf_errno());
 }
 
-bool udp_sock_setup(dns_addr_t *addrconf) {
+bool udp_sock_setup(dns_thread_t* t) {
+    dmn_assert(t);
+
+    dns_addr_t* addrconf = t->ac;
     dmn_assert(addrconf);
 
     const anysin_t* asin = &addrconf->addr;
 
     // mod udp_recv_width down to 1 when unsupported, makes other logic simpler
+    // XXX fix this so addrconf can be const?????
     if((!has_mmsg() || RUNNING_ON_VALGRIND) && addrconf->udp_recv_width > 1)
         addrconf->udp_recv_width = 1;
 
@@ -155,6 +160,12 @@ bool udp_sock_setup(dns_addr_t *addrconf) {
     if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &opt_one, sizeof opt_one) == -1)
         log_fatal("Failed to set SO_REUSEADDR on UDP socket: %s", logf_errno());
 
+#ifdef SO_REUSEPORT
+    if(t->ac->udp_threads > 1)
+        if(setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, &opt_one, sizeof opt_one) == -1)
+            log_fatal("Failed to set SO_REUSEPORT on UDP socket: %s", logf_errno());
+#endif
+
     int opt_size;
     socklen_t size_size = sizeof(opt_size);
 
@@ -226,25 +237,8 @@ bool udp_sock_setup(dns_addr_t *addrconf) {
     else
         udp_sock_opts_v4(sock, gdnsd_anysin_is_anyaddr(asin));
 
-    addrconf->udp_sock = sock;
-
-    if(bind(sock, &asin->sa, asin->len)) {
-        if(errno == EADDRNOTAVAIL) {
-            if(addrconf->autoscan) {
-                log_warn("Could not bind UDP socket %s (%s), configured by automatic interface scanning.  Will ignore this listen address.", logf_anysin(asin), logf_errno());
-                addrconf->udp_autoscan_bind_failed = true;
-                return false;
-            }
-            else if(addrconf->late_bind_secs) {
-                addrconf->udp_need_late_bind = true;
-                log_info("UDP DNS socket %s not yet available, will attempt late bind every %u seconds", logf_anysin(asin), addrconf->late_bind_secs);
-                return ntohs(isv6 ? asin->sin6.sin6_port : asin->sin.sin_port) < 1024 ? true : false;
-            }
-        }
-        log_fatal("Failed to bind() UDP socket to %s: %s", logf_anysin(asin), logf_errno());
-    }
-
-    return false;
+    t->sock = sock;
+    return dnsio_bind(t);
 }
 
 #ifndef MAP_ANONYMOUS
@@ -389,9 +383,9 @@ static void mainloop_mmsg(const unsigned width, const int fd, dnspacket_context_
                 dmn_assert(sent != 0);
                 dmn_assert(sent <= pkts);
                 if(unlikely(sent < pkts)) {
-                    int sockerr;
-                    socklen_t sock_len;
-                    getsockopt(fd, SOL_SOCKET, SO_ERROR, &sockerr, &sock_len);
+                    int sockerr = 0;
+                    socklen_t sock_len = sizeof(sockerr);
+                    (void)getsockopt(fd, SOL_SOCKET, SO_ERROR, &sockerr, &sock_len);
                     stats_own_inc(&pctx->stats->udp.sendfail);
                     if(sent < 0) sent = 0;
                     log_err("UDP sendmmsg() of %li bytes to client %s failed: %s", dgptr[sent].msg_hdr.msg_iov[0].iov_len, logf_anysin(dgptr[sent].msg_hdr.msg_name), logf_errnum(sockerr));
@@ -432,18 +426,19 @@ static void thread_clean(void* unused_arg V_UNUSED) {
 }
 
 F_NORETURN
-void* dnsio_udp_start(void* addrconf_asvoid) {
-    dmn_assert(addrconf_asvoid);
+void* dnsio_udp_start(void* thread_asvoid) {
+    dmn_assert(thread_asvoid);
 
-    const dns_addr_t* addrconf = (const dns_addr_t*) addrconf_asvoid;
+    const dns_thread_t* t = (const dns_thread_t*) thread_asvoid;
+    const dns_addr_t* addrconf = t->ac;
 
-    dnspacket_context_t* pctx = dnspacket_context_new(addrconf->udp_threadnum, true);
+    dnspacket_context_t* pctx = dnspacket_context_new(t->threadnum, true);
 
     pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
 
-    if(addrconf->udp_need_late_bind) {
+    if(t->need_late_bind) {
         const anysin_t* asin = &addrconf->addr;
-        while(bind(addrconf->udp_sock, &asin->sa, asin->len)) {
+        while(bind(t->sock, &asin->sa, asin->len)) {
             if(errno != EADDRNOTAVAIL) {
                 log_err("Failed late bind() of UDP socket to %s: %s.  This listener thread is now shutting down.  Late bind attempts for this socket will no longer be attempted!", logf_anysin(asin), logf_errno());
                 pthread_exit(NULL);
@@ -452,7 +447,7 @@ void* dnsio_udp_start(void* addrconf_asvoid) {
         }
         log_info("Late bind() of UDP socket to %s succeeded, serving requests now", logf_anysin(asin));
     }
-    else if(addrconf->udp_autoscan_bind_failed) {
+    else if(t->autoscan_bind_failed) {
         // already logged this condition back when bind() failed, but it's simpler
         //  to spawn the thread and do the dnspacket_context_new() here properly and
         //  then exit the iothread.  The rest of the code will see this as a thread that
@@ -469,12 +464,12 @@ void* dnsio_udp_start(void* addrconf_asvoid) {
     if(addrconf->udp_recv_width > 1) {
         log_debug("sendmmsg() with a width of %u enabled for UDP socket %s",
             addrconf->udp_recv_width, logf_anysin(&addrconf->addr));
-        mainloop_mmsg(addrconf->udp_recv_width, addrconf->udp_sock, pctx, need_cmsg);
+        mainloop_mmsg(addrconf->udp_recv_width, t->sock, pctx, need_cmsg);
     }
     else
 #endif
     {
-        mainloop(addrconf->udp_sock, pctx, need_cmsg);
+        mainloop(t->sock, pctx, need_cmsg);
     }
 
     pthread_cleanup_pop(1);
