@@ -153,8 +153,6 @@ static const char json_fixed[] =
     "\t\t\"sendfail\": %" PRIuPTR "\r\n"
     "\t}";
 
-static const char json_more[] = ",\r\n";
-static const char json_nl[] = "\r\n";
 static const char json_footer[] = "}\r\n";
 
 static const char html_fixed[] =
@@ -247,7 +245,7 @@ static void populate_stats(void) {
     if(gconfig.realtime_stats || now > pop_statio_time) {
         memset(&statio, 0, sizeof(statio));
 
-        const unsigned nio = gconfig.num_io_threads;
+        const unsigned nio = gconfig.num_dns_threads;
         for(unsigned i = 0; i < nio; i++)
             accumulate_statio(i);
         pop_statio_time = now;
@@ -511,7 +509,7 @@ static void accept_cb(struct ev_loop* loop, ev_io* io, int revents V_UNUSED) {
 
     const int sock = accept(io->fd, &asin->sa, &asin->len);
 
-    if(unlikely(sock == -1)) {
+    if(unlikely(sock < 0)) {
         free(asin);
         switch(errno) {
             case EAGAIN:
@@ -622,7 +620,7 @@ void statio_init(void) {
 
     for(unsigned i = 0; i < num_lsocks; i++) {
         const anysin_t* asin = &gconfig.http_addrs[i];
-        lsocks[i] = tcp_listen_pre_setup(asin, gconfig.http_timeout);
+        lsocks[i] = tcp_listen_pre_setup(asin, gconfig.http_timeout, false);
         if(bind(lsocks[i], &asin->sa, asin->len))
             log_fatal("Failed to bind() stats TCP socket to %s: %s", logf_anysin(asin), logf_errno());
         if(listen(lsocks[i], 128) == -1)
