@@ -34,7 +34,7 @@
 
 #include "ztree.h"
 
-// IFF gconfig.zones_rfc1035_strict_startup is true,
+// IFF gconfig.zones_strict_startup is true,
 //   this flag will be temporarily set to true during
 //   the initial scan, then set back to false, making
 //   zonefile parsing errors fatal for the initial scan.
@@ -323,7 +323,7 @@ F_NONNULL
 static void quiesce_check(struct ev_loop* loop, ev_timer* timer, int revents V_UNUSED) {
     dmn_assert(loop);
     dmn_assert(timer);
-    dmn_assert(revents = EV_TIMER);
+    dmn_assert(revents == EV_TIMER);
 
     zfile_t* zf = (zfile_t*)timer->data;
     dmn_assert(zf->pending_event == timer);
@@ -727,9 +727,18 @@ static void inot_reader(struct ev_loop* loop, ev_io* w, int revents V_UNUSED) {
 
         unsigned offset = 0;
         while(offset < (unsigned)bytes) {
-            dmn_assert((bytes - offset) >= sizeof(struct inotify_event));
+            if((bytes - offset) < sizeof(struct inotify_event)) {
+                log_err("rfc1035: inotify sent truncated/garbage data");
+                handle_inotify_failure(loop);
+                return;
+            }
             struct inotify_event* evt = (void*)&evtbuf[offset];
             offset += sizeof(struct inotify_event);
+            if((bytes - offset) < evt->len) {
+                log_err("rfc1035: inotify sent truncated/garbage data");
+                handle_inotify_failure(loop);
+                return;
+            }
             offset += evt->len;
             if(inot_process_event(loop, (evt->len > 0 ? evt->name : NULL), evt->mask))
                 return;
@@ -852,7 +861,7 @@ void zsrc_rfc1035_load_zones(void) {
     set_quiesce();
     if(gconfig.zones_rfc1035_auto)
         inotify_initial_setup(); // no-op if no compile-time support
-    if(gconfig.zones_rfc1035_strict_startup)
+    if(gconfig.zones_strict_startup)
         fail_fatally = true;
     struct ev_loop* temp_load_loop = ev_loop_new(EVFLAG_AUTO);
     scan_dir(temp_load_loop, min_quiesce);
