@@ -276,9 +276,15 @@ static bool gdmap_update_nets(gdmap_t* gdmap) {
     const vscf_data_t* nets_cfg = vscf_scan_filename(gdmap->nets_path, &vscf_err);
     nlist_t* new_list = NULL;
     if(nets_cfg) {
-        new_list = nets_make_list(nets_cfg, update_dclists, gdmap->name);
-        if(!new_list)
-            log_err("plugin_geoip: map '%s': (Re-)loading nets file '%s' failed!", gdmap->name, logf_pathname(gdmap->nets_path));
+        if(vscf_is_hash(nets_cfg)) {
+            new_list = nets_make_list(nets_cfg, update_dclists, gdmap->name);
+            if(!new_list)
+                log_err("plugin_geoip: map '%s': (Re-)loading nets file '%s' failed!", gdmap->name, logf_pathname(gdmap->nets_path));
+        }
+        else {
+            dmn_assert(vscf_is_array(nets_cfg));
+            log_err("plugin_geoip: map '%s': (Re-)loading nets file '%s' failed: file cannot be an array of values", gdmap->name, logf_pathname(gdmap->nets_path));
+        }
         vscf_destroy(nets_cfg);
     }
     else {
@@ -668,36 +674,43 @@ const char* gdmaps_dcnum2name(const gdmaps_t* gdmaps, const unsigned gdmap_idx,
 }
 
 // mostly for debugging / error output
-#define DCLIST_LOGF_MAX 512
-static const char dclist_len_err[] = "<dclist too large to format for printing>";
+// Note that this doesn't participate in liburcu stuff, and therefore could crash if it were
+//   running concurrently with an update swap.  It's only used from the testsuite and gdmaps_geoip_test
+//   stuff, though, so that's not important.
 static const char dclist_nodc[] = "<INVALID>";
 const char* gdmaps_logf_dclist(const gdmaps_t* gdmaps, const unsigned gdmap_idx, const uint8_t* dclist) {
     dmn_assert(gdmaps); dmn_assert(dclist);
     dmn_assert(gdmap_idx < gdmaps->count);
 
-    char tbuf[DCLIST_LOGF_MAX];
-    tbuf[0] = '\0';
-    unsigned tbuf_remain = DCLIST_LOGF_MAX - 1;
+    // Save original...
+    const uint8_t* dclist_orig = dclist;
 
+    // Size the output
+    unsigned output_len = 0;
     unsigned dcnum;
     bool first = true;
     while((dcnum = *dclist++)) {
         const char* dcname = gdmaps_dcnum2name(gdmaps, gdmap_idx, dcnum);
-        if(!dcname)
-            dcname = dclist_nodc;
-        unsigned addlen = strlen(dcname);
-        if(!first) addlen += 2;
-        if(addlen > tbuf_remain)
-            return dclist_len_err;
+        output_len += strlen(dcname ? dcname : dclist_nodc);
+        if(!first) output_len += 2;
+        first = false;
+    }
+
+    // Allocate buffer
+    char* buf = dmn_fmtbuf_alloc(output_len + 1);
+    buf[0] = '\0';
+
+    // Actually write the output
+    first = true;
+    dclist = dclist_orig;
+    while((dcnum = *dclist++)) {
+        const char* dcname = gdmaps_dcnum2name(gdmaps, gdmap_idx, dcnum);
         if(!first)
-            strcat(tbuf, ", ");
-        strcat(tbuf, dcname);
-        tbuf_remain -= addlen;
+            strcat(buf, ", ");
+        strcat(buf, dcname ? dcname : dclist_nodc);
         first = false;
     }
 
-    char* buf = dmn_fmtbuf_alloc(strlen(tbuf) + 1);
-    strcpy(buf, tbuf);
     return buf;
 }
 
