@@ -2,10 +2,8 @@
 
 set -e
 
-GENERIC_HELP="You can either fix this and re-run configure, or use other means to obtain and decompress the 5 data files at 'http://gdnsd.net/GeoLiteTestData/', placing them in the build directory '$ABDIR'"
-
 if [ $GEOLITE_DECOMP = "none" ]; then
-    echo "xz not found in PATH by configure.  $GENERIC_HELP"
+    echo "xz not found in PATH by configure"
     exit 1
 elif [ $GEOLITE_DECOMP = "xz" ]; then
     compext=.xz
@@ -15,7 +13,7 @@ else
 fi
 
 if [ $GEOLITE_DL = "none" ]; then
-    echo "Suitable downloader (wget or curl) not found in PATH by configure.  $GENERIC_HELP"
+    echo "Suitable downloader (wget or curl) not found in PATH by configure"
     exit 1
 fi
 
