@@ -39,8 +39,79 @@ use File::Copy qw//;
 use Socket qw/AF_INET/;
 use Socket6 qw/AF_INET6 inet_pton/;
 use IO::Socket::INET6 qw//;
+use File::Path qw//;
 use Config;
 
+# The generic rr-matching code assumes "$rr->string eq $rr->string" is sufficient
+#   to detect a test-failure difference between two RRs.  However, the stock ->string
+#   code in Net::DNS::RR::OPT doesn't actually print the RDATA portion of OPT RRs,
+#   which is where the option code + data live for e.g. edns-client-subnet.  Further,
+#   even with ->string fixed, it was discovered that ->new wasn't actually decoding
+#   the option data properly anyways.  Both are hacked below to make our test comparisons
+#   actually detect failures...
+# Arguably we could just patch our local copy directly instead of hacking methods,
+#   but I'd rather keep our copy of Net::DNS relatively clean for now, pending whatever
+#   comes of all that.
+use Net::DNS::RR::OPT;
+{
+    no warnings 'redefine';
+
+    # The only difference here is removal of the buggy 'unpack("n",...)' from
+    #  the decoding of "optiondata"
+    *Net::DNS::RR::OPT::new = sub {
+	    my ($class, $self, $data, $offset) = @_;
+
+	    $self->{"name"} = "" ;   # should allway be "root"
+	    if ($self->{"rdlength"} > 0) {
+		    $self->{"optioncode"}   = unpack("n", substr($$data, $offset, 2));
+		    $self->{"optionlength"} = unpack("n", substr($$data, $offset+2, 2));
+		    $self->{"optiondata"}   = substr($$data, $offset+4, $self->{"optionlength"});
+	    }
+
+	    $self->{"_rcode_flags"}  = pack("N",$self->{"ttl"});
+
+	    $self->{"extendedrcode"} = unpack("C", substr($self->{"_rcode_flags"}, 0, 1));
+	    $self->{"ednsversion"}   = unpack("C", substr($self->{"_rcode_flags"}, 1, 1));
+	    $self->{"ednsflags"}     = unpack("n", substr($self->{"_rcode_flags"}, 2, 2));
+
+	    return bless $self, $class;
+    };
+
+    # This was changed to actually reflect optioncode/data differences in the string output
+    *Net::DNS::RR::OPT::string = sub {
+        my $self = shift;
+        my $basics =
+            "; EDNS Version "     . $self->{"ednsversion"} .
+            "\t UDP Packetsize: " .  $self->{"class"} .
+            "\n; EDNS-RCODE:\t"   . $self->{"extendedrcode"} .
+            " (" . $Net::DNS::RR::OPT::extendedrcodesbyval{ $self->{"extendedrcode"} }. ")" .
+            "\n; EDNS-FLAGS:\t"   . sprintf("0x%04x", $self->{"ednsflags"}) .
+            "\n";
+        # Technically, there can be multiple options, but the rest of Net::DNS::RR:OPT
+        #  just assumes one and ignores any others, and that's all we really need for now anyways
+        my $optdata = "";
+        if($self->{"optioncode"}) {
+            $optdata =
+                "; EDNS Option Code: " . $self->{"optioncode"} .
+                "\n; EDNS Option Len: " . $self->{"optionlength"} .
+                "\n; EDNS Option Hex Data: " . unpack('H*', $self->{"optiondata"}) .
+                "\n";
+        }
+
+        return $basics . $optdata;
+    };
+}
+
+sub safe_rmtree {
+    my $target = shift;
+    if(-e $target) {
+        File::Path::rmtree($target, 0, 1);
+        if(-e $target) {
+            die "Could not rmtree '$target': $!";
+        }
+    }
+}
+
 my %SIGS;
 {
     my $i = 0;
@@ -52,20 +123,14 @@ my %SIGS;
 
 # Set up per-testfile output directory and zones input directory
 our $OUTDIR;
-our $ZONES_IN;
-our $ALTZONES_IN;
+our $ALTZONES_IN = $FindBin::Bin . '/altzones/';
 {
     my $tname = $FindBin::Bin;
     $tname =~ s{^.*/}{};
     $tname .= '_' . $FindBin::Script;
     $tname =~ s{\.t$}{};
 
-    $ZONES_IN = $FindBin::Bin . '/zones/';
-    $ALTZONES_IN = $FindBin::Bin . '/altzones/';
     $OUTDIR = $ENV{TESTOUT_DIR} . '/' . $tname;
-    foreach my $d ($OUTDIR, "$OUTDIR/etc", "$OUTDIR/etc/zones") {
-        mkdir $d unless -d $d
-    }
 }
 
 # this makes gdnsd start faster on low-res filesystems,
@@ -100,10 +165,20 @@ our $HAVE_V6 = 1;
     }
 }
 
+# If user runs testsuite as root, we try to set the privdrop
+#   user to nobody as a more-reliable choice.  Failing that,
+#   hopefully they read the large warning at the start of
+#   the testsuite output...
+our $PRIVDROP_USER = ($> == 0) ? 'nobody' : '';
+
 our $GDNSD_BIN = $ENV{INSTALLCHECK_SBINDIR}
     ? "$ENV{INSTALLCHECK_SBINDIR}/gdnsd"
     : "$ENV{TOP_BUILDDIR}/gdnsd/gdnsd";
 
+our $EXTMON_BIN = $ENV{INSTALLCHECK_BINDIR}
+    ? "$ENV{INSTALLCHECK_BINDIR}/gdnsd_extmon_helper"
+    : "$ENV{TOP_BUILDDIR}/plugins/extmon/gdnsd_extmon_helper";
+
 # During installcheck, the default hardcoded plugin path
 #  should work correctly for finding the installed plugins
 our $PLUGIN_PATH;
@@ -125,7 +200,7 @@ else {
         )
         . q{"]};
     closedir($dh);
-    $EXTMON_HELPER_CFG = qq|extmon => { helper_path => "$ENV{TOP_BUILDDIR}/plugins/extmon/gdnsd_extmon_helper" }|;
+    $EXTMON_HELPER_CFG = qq|extmon => { helper_path => "$EXTMON_BIN" }|;
 }
 
 our $RAND_LOOPS = $ENV{GDNSD_RTEST_LOOPS} || 100;
@@ -239,27 +314,13 @@ sub check_stats {
     }
 }
 
-our $GDOUT_FH; # open fh on gdnsd.out for test_log_output()
-our $INOTIFY_ENABLED = 0;
+sub proc_tmpl {
+    my ($inpath, $outpath) = @_;
 
-sub spawn_daemon {
-    my ($class, $cfgfile, $geoip_data) = @_;
-
-    my $daemon_out = $OUTDIR . '/gdnsd.out';
-    my $cfgout = $OUTDIR . '/etc/config';
-    my $zonesout = $OUTDIR . '/etc/zones/';
-
-    if($geoip_data) {
-        require _FakeGeoIP;
-        mkdir "$OUTDIR/etc/geoip" unless -d "$OUTDIR/etc/geoip";
-        my $geoip_out = $OUTDIR . '/etc/geoip/FakeGeoIP.dat';
-        _FakeGeoIP::make_fake_geoip($geoip_out, $geoip_data);
-    }
-
-    open(my $orig_fh, '<', $cfgfile)
-        or die "Cannot open test configfile '$cfgfile' for reading: $!";
-    open(my $out_fh, '>', $cfgout)
-        or die "Cannot open test config text output '$cfgout' for writing: $!";
+    open(my $in_fh, '<', $inpath)
+        or die "Cannot open test template '$inpath' for reading: $!";
+    open(my $out_fh, '>', $outpath)
+        or die "Cannot open test template output '$outpath' for writing: $!";
 
     my $dns_lspec = $HAVE_V6
         ? qq{[ 127.0.0.1, ::1 ]}
@@ -269,41 +330,104 @@ sub spawn_daemon {
         ? qq{[ 127.0.0.1, ::1 ]}
         : qq{127.0.0.1};
 
-    while(<$orig_fh>) {
+    if($PRIVDROP_USER) {
+        $dns_lspec .= "\nusername = $PRIVDROP_USER";
+    }
+
+    while(<$in_fh>) {
         s/\@dns_lspec\@/$dns_lspec/g;
         s/\@http_lspec\@/$http_lspec/g;
         s/\@dns_port\@/$DNS_PORT/g;
         s/\@http_port\@/$HTTP_PORT/g;
         s/\@extra_port\@/$EXTRA_PORT/g;
         s/\@pluginpath\@/$PLUGIN_PATH/g;
-        s/\@extmon_helper_cfg\@/$EXTMON_HELPER_CFG/g;
+        # if the test used the extmon helper, pre-execute
+        #   it now to do libtool stuff before privdrop, in case
+        #   of testsuite running as root.  Otherwise on first
+        #   execution libtool tries to write to the builddir as
+        #   as the privdrop user.
+        if(s/\@extmon_helper_cfg\@/$EXTMON_HELPER_CFG/g && $PRIVDROP_USER) {
+            system("$EXTMON_BIN >/dev/null 2>&1");
+        }
         print $out_fh $_;
     }
-    close($orig_fh) or die "Cannot close test configfile '$cfgfile': $!";
-    close($out_fh) or die "Cannot close test config text output file '$cfgout': $!";
 
-    # Wipe zones, in case of dynamic stuff from earlier run
-    opendir(my $zdh, $zonesout) or die "Cannot open zones output subdirectory: $!";
-    foreach my $zf (grep { !/^\./ } readdir($zdh)) {
-        unlink($zonesout . $zf)
-            or die "Cannot unlink zonefile '$_': $!";
-    }
-    closedir($zdh);
+    close($in_fh) or die "Cannot close test template '$inpath': $!";
+    close($out_fh) or die "Cannot close test template output '$outpath': $!";
+}
 
-    opendir(my $dh, $ZONES_IN) or die "Cannot open zones subdirectory: $!";
-    my @zfiles_list = grep { !/^\./ && ! -d $_ } readdir($dh);
-    closedir($dh);
-    foreach my $zfile (@zfiles_list) {
-        File::Copy::copy("$ZONES_IN/$zfile", $zonesout)
-            or die "Failed to copy zonefile '$zfile' to test area: $!";
+sub recursive_templated_copy {
+    my ($indir, $outdir) = @_;
+    if(opendir(my $dh, $indir)) {
+        if(!-d $outdir) {
+            mkdir $outdir or die "Cannot create directory $outdir: $!";
+        }
+        my @files_list = grep { !/^\./ && ! -d $_ } readdir($dh);
+        closedir($dh);
+        foreach my $f (@files_list) {
+            my $in_full = "$indir/$f";
+            my $out_full = "$outdir/$f";
+            if(-f $in_full) {
+                if($f =~ /^(.*)\.tmpl$/) {
+                    proc_tmpl($in_full, "$outdir/$1");
+                }
+                else {
+                    File::Copy::copy($in_full, $out_full)
+                        or die "Failed to copy '$in_full' to '$out_full': $!";
+                }
+            }
+            elsif(-d $in_full) {
+                recursive_templated_copy($in_full, $out_full);
+            }
+            else {
+                die "Test data file is not a regular file or directory: '$in_full'";
+            }
+        }
     }
+}
+
+our $GDOUT_FH; # open fh on gdnsd.out for test_log_output()
+our $INOTIFY_ENABLED = 0;
+
+sub daemon_abort {
+    my $log_fn = shift;
+    open(my $gdout_fh, '<', $log_fn)
+        or die "gdnsd failed to finish starting properly, and no output file could be found";
+    my $gdout = '';
+    while(<$gdout_fh>) { $gdout .= $_ }
+    close($gdout_fh);
+    die "gdnsd failed to finish starting properly.  output (if any):\n" . $gdout;
+}
+
+sub spawn_daemon {
+    my ($class, $etcsrc, $geoip_data) = @_;
+
+    $etcsrc ||= "etc";
+
+    safe_rmtree($OUTDIR);
+
+    foreach my $d ($OUTDIR, "${OUTDIR}/etc") {
+        mkdir $d or die "Cannot create directory $d: $!";
+    }
+
+    if($geoip_data) {
+        require _FakeGeoIP;
+        my $gdir = "$OUTDIR/etc/geoip";
+        mkdir $gdir or die "Cannot create directory $gdir: $!";
+        _FakeGeoIP::make_fake_geoip($gdir . "/FakeGeoIP.dat", $geoip_data);
+    }
+
+    recursive_templated_copy("${FindBin::Bin}/${etcsrc}", "${OUTDIR}/etc");
 
     my $exec_line = $TEST_RUNNER
         ? qq{$TEST_RUNNER $GDNSD_BIN -d $OUTDIR startfg}
         : qq{$GDNSD_BIN -d $OUTDIR startfg};
 
+    my $daemon_out = $OUTDIR . '/gdnsd.out';
+
     my $pid = fork();
     die "Fork failed!" if !defined $pid;
+
     if(!$pid) { # child, exec daemon
         open(STDOUT, '>', $daemon_out)
             or die "Cannot open '$daemon_out' for writing as STDOUT: $!";
@@ -329,8 +453,9 @@ sub spawn_daemon {
                 or die "Cannot open '$daemon_out' for reading: $!";
             my $is_listening;
             while(<$GDOUT_FH>) {
-                $INOTIFY_ENABLED = 1 if /\Qrfc1035: will use inotify for zone change detection\E$/;
-                return $pid if /\QDNS listeners started\E$/;
+                $INOTIFY_ENABLED = 1 if /\brfc1035: will use inotify for zone change detection$/;
+                daemon_abort($daemon_out) if /\bfatal: /; # don't wait around if we see a fatal log entry...
+                return $pid if /\bDNS listeners started$/;
             }
             close($GDOUT_FH)
                 or die "Cannot close '$daemon_out' for reading: $!";
@@ -338,12 +463,7 @@ sub spawn_daemon {
         select(undef, undef, undef, $retry_delay);
     }
 
-    open(my $gdout_fh, '<', $daemon_out)
-        or die "gdnsd failed to finish starting properly, and no output file could be found";
-    my $gdout = '';
-    while(<$gdout_fh>) { $gdout .= $_ }
-    close($gdout_fh);
-    die "gdnsd failed to finish starting properly.  output (if any):\n" . $gdout;
+    daemon_abort($daemon_out);
 }
 
 sub test_spawn_daemon {
